@@ -1,15 +1,12 @@
 package Tk::DragDrop::Win32Site;
 
 use vars qw($VERSION);
-$VERSION = '4.002'; # $Id: //depot/Tkutf8/DragDrop/Win32Site/Win32Site.pm#2 $
+$VERSION = sprintf '4.%03d', q$Revision: #7 $ =~ /\D(\d+)\s*$/;
 
 use Tk qw($XS_VERSION);
 require DynaLoader;
 require Tk::DropSite;
 
-use vars qw($VERSION);
-$VERSION = '4.002'; # $Id: //depot/Tkutf8/DragDrop/Win32Site/Win32Site.pm#2 $
-
 use base qw(Tk::DropSite DynaLoader);
 
 bootstrap Tk::DragDrop::Win32Site;
@@ -31,20 +28,22 @@ sub InitSite
 
 sub Win32Drop
 {
- print join(',',@_),"\n";
+ # print join(',',@_),"\n";
  my ($w,$site,$msg,$wParam,$lParam) = @_;
  my ($x,$y,@files) = DropInfo($wParam);
  my $cb = $site->{'-dropcommand'};
+ $site->Apply(-entercommand => $x, $y, 0);
  if ($cb)
   {
    foreach my $file (@files)
     {
-     print "$file @ $x,$y\n";
+     # print "$file @ $x,$y\n";
      $w->clipboardClear;
      $w->clipboardAppend('--',$file);
      $cb->Call('CLIPBOARD',$x,$y);
     }
   }
+ $site->Apply(-entercommand => $x, $y, 1);
  return 0;
 }
 
