@@ -15,7 +15,6 @@ use     AutoLoader qw(AUTOLOAD);
 use     DynaLoader;
 use     Cwd();
 use base qw(Exporter DynaLoader);
-use     File::Spec qw();
 
 *fileevent = \&Tk::Event::IO::fileevent;
 
@@ -63,16 +62,13 @@ use strict;
 use Carp;
 
 # Record author's perforce depot record
-#$Tk::CHANGE      = q$Change: 3279 $;
-#$Tk::CHANGE      = 'sfsvn-' . q$Change: 27 $;
-$Tk::CHANGE      = 'git-controlled';
+$Tk::CHANGE      = q$Change: 3215 $;
 
 # $tk_version and $tk_patchLevel are reset by pTk when a mainwindow
 # is created, $VERSION is checked by bootstrap
 $Tk::version     = '8.4';
 $Tk::patchLevel  = '8.4';
-$Tk::VERSION     = '804.035_501';
-$Tk::VERSION     =~ s{_}{};
+$Tk::VERSION     = '804.026';
 $Tk::XS_VERSION  = $Tk::VERSION;
 $Tk::strictMotif = 0;
 
@@ -94,7 +90,7 @@ Preload(DynaLoader::dl_findfile('-L/usr/openwin/lib','-lX11'))
   if (NeedPreload() && -d '/usr/openwin/lib');
 
 use Tk::Submethods ('option'    =>  [qw(add get clear readfile)],
-                    'clipboard' =>  [qw(clear append get)]
+                    'clipboard' =>  [qw(clear append)]
                    );
 
 #
@@ -297,7 +293,7 @@ sub MessageBox {
 	}
         my $md = $parent->Dialog(%$args);
         my $an = $md->Show;
-        $md->destroy if Tk::Exists($md);
+        $md->destroy;
         return $an;
     }
 } # end messageBox
@@ -318,29 +314,15 @@ sub messageBox
  $args{'-default'} = lc($args{'-default'}) if (exists $args{'-default'});
  ucfirst tk_messageBox(-parent => $widget, %args);
 }
-sub _adapt_path_to_os
-{
- # adapting the path of -initalfile and -initialdir to the operating system
- # (like that getOpenFile(-initialdir => 'c:/WINNT') will work, as it will
- #  be converted to c:\WINNT)
- my %args = @_;
- foreach my $option (qw(-initialfile -initialdir))
-  {
-   if ($args{$option})
-    {
-     $args{$option} = File::Spec->catfile($args{$option});
-    }
-   }
- return %args;
-}    
+
 sub getOpenFile
 {
- tk_getOpenFile(-parent => shift,_adapt_path_to_os(@_));
+ tk_getOpenFile(-parent => shift,@_);
 }
 
 sub getSaveFile
 {
- tk_getSaveFile(-parent => shift,_adapt_path_to_os(@_));
+ tk_getSaveFile(-parent => shift,@_);
 }
 
 sub chooseColor
@@ -350,7 +332,7 @@ sub chooseColor
 
 sub chooseDirectory
 {
- tk_chooseDirectory(-parent => shift,_adapt_path_to_os(@_));
+ tk_chooseDirectory(-parent => shift,@_);
 }
 
 sub DialogWrapper
@@ -551,16 +533,6 @@ sub FocusChildren { shift->children }
 # to the next window after this one in the traversal
 # order.
 sub focusNext
-{
- my $w = shift;
- my $cur = $w->getNextFocus;
- if ($cur)
-  {
-   $cur->tabFocus;
-  }
-}
-
-sub getNextFocus
 {
  my $w = shift;
  my $cur = $w;
@@ -591,7 +563,8 @@ sub getNextFocus
     }
    if ($cur == $w || $cur->FocusOK)
     {
-     return $cur;
+     $cur->tabFocus;
+     return;
     }
   }
 }
@@ -606,16 +579,6 @@ sub getNextFocus
 # to the previous window before this one in the traversal
 # order.
 sub focusPrev
-{
- my $w = shift;
- my $cur = $w->getPrevFocus;
- if ($cur)
-  {
-   $cur->tabFocus;
-  }
-}
-
-sub getPrevFocus
 {
  my $w = shift;
  my $cur = $w;
@@ -655,7 +618,8 @@ sub getPrevFocus
    $cur = $parent;
    if ($cur == $w || $cur->FocusOK)
     {
-     return $cur;
+     $cur->tabFocus;
+     return;
     }
   }
 
