@@ -1,11 +1,11 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2004 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::After;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '4.003'; # $Id: //depot/Tkutf8/Tk/After.pm#3 $
+$VERSION = '4.007'; # $Id: //depot/Tkutf8/Tk/After.pm#10 $
 
 sub _cancelAll
 {
@@ -15,9 +15,12 @@ sub _cancelAll
   {
    # carp "Auto cancel ".$obj->[1]." for ".$obj->[0]->PathName;
    $obj->cancel;
+   bless $obj,"Tk::After::Cancelled";
   }
 }
 
+sub Tk::After::Cancelled::once { }
+
 sub submit
 {
  my $obj     = shift;
@@ -40,7 +43,9 @@ sub submit
 sub DESTROY
 {
  my $obj = shift;
- @{$obj} = ();
+ $obj->cancel;
+ undef $obj->[0];
+ undef $obj->[4];
 }
 
 sub new
@@ -58,7 +63,7 @@ sub cancel
  my $w   = $obj->[0];
  if ($id)
   {
-   $w->Tk::after('cancel'=> $id);
+   $w->Tk::after('cancel'=> $id) if Tk::Exists($w);
    delete $w->{_After_}{$id} if exists $w->{_After_};
    $obj->[1] = undef;
   }
@@ -83,6 +88,16 @@ sub once
  $obj->[4]->Call;
 }
 
+sub time {
+    my $obj = shift;
+    my $delay = shift;
+    if (defined $delay) {
+	$obj->cancel if $delay == 0;
+	$obj->[2] = $delay;
+    }
+    $obj->[2];
+}
+
 1;
 __END__
 
