@@ -8,7 +8,7 @@ BEGIN { @MainWindow::ISA = 'Tk::MainWindow' }
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '4.013'; # was: sprintf '4.%03d', q$Revision: #12 $ =~ /\D(\d+)\s*$/;
+$VERSION = sprintf '4.%03d', q$Revision: #11 $ =~ /\D(\d+)\s*$/;
 
 use Tk::CmdLine;
 use Tk qw(catch);
@@ -153,7 +153,7 @@ sub Existing
 
 END
 {
- if (Tk::IsParentProcess())
+ if ($pid == $$)
   {
    foreach my $top (values %Windows)
     {
@@ -210,3 +210,4 @@ sub WMSaveYourself
 
 __END__
 
+=cut
