@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000 Nick Ing-Simmons. All rights reserved.
+  Copyright (c) 2000-2004 Nick Ing-Simmons. All rights reserved.
   This program is free software; you can redistribute it and/or
   modify it under the same terms as Perl itself.
 */
@@ -8,6 +8,11 @@
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
+
+#ifdef HAS_NL_LANGINFO
+#include <langinfo.h>
+#endif
+
 #define U8 U8
 #include "tkGlue.def"
 
@@ -65,6 +70,13 @@ Tcl_UniCharIsAlpha(int ch)
  return Perl_is_uni_alpha(aTHX_ ch);
 }
 
+int
+Tcl_UniCharIsSpace(int ch)
+{
+ dTHX;
+ return Perl_is_uni_space(aTHX_ ch);
+}
+
 int
 Tcl_UniCharIsUpper(int ch)
 {
@@ -136,7 +148,11 @@ Tcl_UniCharToUtf(int ch, char * buf)
 {
  dTHX;
  /* We "allow any" as the page cache algorithm hits at least U+FFFE */
+#ifdef UNICODE_ALLOW_ANY
  U8 *p = uvchr_to_utf8_flags((U8 *) buf,ch, UNICODE_ALLOW_ANY);
+#else
+ U8 *p = Perl_uv_to_utf8(aTHX_ (U8 *) buf,ch);
+#endif
  return p - (U8 *) buf;
 }
 
@@ -525,13 +541,24 @@ GetSystemEncoding(void)
 {
  if (!system_encoding)
   {
-   system_encoding = Tcl_GetEncoding(NULL,"iso8859-1");
+   char *codeset = "iso8859-1";
+/* This assumes perl's Configure probe stuff is #include-d above */
+#if defined(HAS_NL_LANGINFO) && defined(CODESET)
+   codeset = nl_langinfo(CODESET);
+#endif
+   system_encoding = Tcl_GetEncoding(NULL,codeset);
   }
  return system_encoding;
 }
 
 #define PerlEncObj(enc) (HeVAL((HE *) (enc)))
 
+SV *
+Lang_SystemEncoding(void)
+{
+ return PerlEncObj(GetSystemEncoding());
+}
+
 Tcl_Encoding
 Tcl_GetEncoding (Tcl_Interp * interp, CONST char * name)
 {
@@ -539,14 +566,12 @@ Tcl_GetEncoding (Tcl_Interp * interp, CONST char * name)
  HE *he;
  STRLEN len = strlen(name);
  SV *sv   = NULL;
- SV *nsv  = newSVpv((char *)name,len);
- I32 hash;
- PERL_HASH(hash, (char *)name, len);
+ SV *nmsv  = newSVpv((char *)name,len);
  if (!encodings)
   {
    encodings = newHV();
   }
- he = hv_fetch_ent(encodings,nsv,0,hash);
+ he = hv_fetch_ent(encodings,nmsv,0,0);
  if (!he || !HeVAL(he))
   {
    dSP;
@@ -554,19 +579,19 @@ Tcl_GetEncoding (Tcl_Interp * interp, CONST char * name)
    SAVETMPS;
    PUSHMARK(sp);
    XPUSHs(sv_2mortal(newSVpv("Tk",0)));
-   XPUSHs(nsv);
+   XPUSHs(nmsv);
    PUTBACK;
    perl_call_method("getEncoding",G_SCALAR);
    SPAGAIN;
    sv = POPs;
    PUTBACK;
-   he = hv_store_ent(encodings,nsv,newSVsv(sv),hash);
+   he = hv_store_ent(encodings,nmsv,newSVsv(sv),0);
    if (0 && !SvOK(sv))
     warn("Cannot find '%s'",name);
    FREETMPS;
    LEAVE;
   }
- SvREFCNT_dec(nsv);
+ SvREFCNT_dec(nmsv);
  sv = HeVAL(he);
  if (sv_isobject(sv))
   {
@@ -616,28 +641,14 @@ Tcl_GetEncodingName(Tcl_Encoding encoding)
  return HePV(he,len);
 }
 
-int
-Tcl_ExternalToUtf (Tcl_Interp * interp,
-				Tcl_Encoding encoding, CONST char * src,
-				int srcLen, int flags,
-				Tcl_EncodingState * statePtr, char * dst,
-				int dstLen, int * srcReadPtr,
-				int * dstWrotePtr, int * dstCharsPtr)
-{
- if (!encoding)
-  encoding = GetSystemEncoding();
- LangDebug(__FUNCTION__ " %s\n",Tcl_GetEncodingName(encoding));
- croak(__FUNCTION__ " not implemented");
- return TCL_ERROR;
-}
-
-int
-Tcl_UtfToExternal(Tcl_Interp * interp,
-				Tcl_Encoding encoding, CONST char * src,
-				int srcLen, int flags,
-				Tcl_EncodingState * statePtr, char * dst,
-				int dstLen, int * srcReadPtr,
-				int * dstWrotePtr, int * dstCharsPtr)
+static int
+CallEncode(Tcl_Interp * interp,
+	   Tcl_Encoding encoding, CONST char * src,
+	   int srcLen, int flags,
+	   Tcl_EncodingState * statePtr, char * dst,
+	   int dstLen, int * srcReadPtr,
+	   int * dstWrotePtr, int * dstCharsPtr,
+	   const char *method)
 {
  dTHX;
  int srcRead;
@@ -650,76 +661,89 @@ Tcl_UtfToExternal(Tcl_Interp * interp,
  U8 *dend;
  int chars = 0;
  dSP;
- SV *quiet = get_sv("Tk::encode_fallback",0);
+ SV *quiet;
  SV *stmp;
  SV *dtmp;
  char *td;
  STRLEN dbytes;
+ if (flags & TCL_ENCODING_STOPONERROR)
+  quiet = get_sv("Tk::encodeStopOnError",0);
+ else
+  quiet = get_sv("Tk::encodeFallback",0);
  if (!encoding)
   encoding = GetSystemEncoding();
  if (!sv_isobject(PerlEncObj(encoding)))
   abort();
- if (0 && encoding)
-  LangDebug(__FUNCTION__ " %s\n",Tcl_GetEncodingName(encoding));
  if (!srcReadPtr)
   srcReadPtr = &srcRead;
  if (!dstWrotePtr)
   dstWrotePtr = &dstWrote;
  if (!dstCharsPtr)
   dstCharsPtr = &dstChars;
+ else
+  {
+   LangDebug("%s wants char count\n",method);
+  }
  if (!src)
   srcLen = 0;
  if (srcLen < 0)
   srcLen = strlen(src);
  send = s+srcLen;
- dstLen--;
+ dstLen -= 2;
  dend = d + dstLen;
- stmp = newSV(4);
- if (s == send)
-  {
-   LangDebug(__FUNCTION__ "Nothing to convert\n");
-  }
+ stmp = newSV(srcLen);
  while (s < send)
   {
-#ifdef SvUTF8
-#if defined(utf8n_to_uvchr)
-   STRLEN len;
-   UV ch = utf8n_to_uvchr(s, send-s, &len, UTF8_ALLOW_ANY|UTF8_CHECK_ONLY);
-#else
-   I32 len;
-   UV ch = utf8_to_uv(s,&len);
-#endif
-   if ((I32) len < 0)
+   STRLEN len = srcLen;
+   if (*method == 'e')
     {
-     LangDebug("%.*s => Syntax\n",srcLen,s);
-     code = TCL_CONVERT_SYNTAX;
-     break;
-    }
-#else
-   I32 len = 1;
-   UV ch   = UCHAR(*s);
+#if 0
+     /* We used to do things one char at a time ... can't remember why
+        perhaps to handle partial chars ?
+        we got perl to tell us length of one char using call below
+        Only makes sense for encode when source is UTF-8, though
+        by luck it worked for "decode" of UTF-8 as well
+        provided we did not set SvUTF8_on which upset Encode.xs
+      */
+     UV ch = utf8n_to_uvchr(s, send-s, &len, UTF8_ALLOW_ANY|UTF8_CHECK_ONLY);
 #endif
+     sv_setpvn(stmp,s,len);
+     if (has_highbit(s,len))
+      SvUTF8_on(stmp);
+    }
+   else
+    {
+     sv_setpvn(stmp,s,len);
+    }
    PUSHMARK(sp);
    XPUSHs(PerlEncObj(encoding));
-   sv_setpvn(stmp,s,len);
-#ifdef SvUTF8
-   if (has_highbit(s,len))
-    SvUTF8_on(stmp);
-#endif
    XPUSHs(stmp);
    XPUSHs(quiet);
    PUTBACK;
-   perl_call_method("encode",G_SCALAR);
+   perl_call_method(method,G_SCALAR|G_EVAL);
+   if (SvTRUE(ERRSV))
+    {
+     code = TCL_ERROR;
+     if (interp)
+      {
+       Tcl_SetResult(interp,SvPV_nolen(ERRSV),TCL_VOLATILE);
+      }
+     else
+      {
+       warn("%_",ERRSV);
+      }
+     break;
+    }
    SPAGAIN;
    dtmp = POPs;
    PUTBACK;
    if (SvCUR(stmp))
     {
+     /* This could also be TCL_CONVERT_MULTIBYTE - how do we tell ? */
      code = TCL_CONVERT_UNKNOWN;
      break;
     }
    td = SvPV(dtmp,dbytes);
-
    if (!dbytes)
     {
      code = TCL_CONVERT_UNKNOWN;
@@ -728,26 +752,55 @@ Tcl_UtfToExternal(Tcl_Interp * interp,
    if (d+dbytes > dend)
     {
      code = TCL_CONVERT_NOSPACE;
+     dbytes = dend-d;
      break;
     }
    memcpy(d,td,dbytes);
    d += dbytes;
+   /* FIXME? : Char count is bogus unless we do one-at-atime - if
+      we find something that wants it we need to get it some
+      other way - e.g. UTF8_SKIP()ing over whichever of src/dst is UTF-8
+    */
    chars++;
    s += len;
   }
  SvREFCNT_dec(stmp);
  *srcReadPtr  = (s - (U8 *)src);
  *dstCharsPtr = chars;
- dst[dstLen]  = '\0';
+ dst[dstLen]   = '\0';
+ dst[dstLen+1]   = '\0';
+ /* If dest is wide single '\0' may not be enough */
+ Zero(d,dend-d,char);
  *dstWrotePtr = (d- (U8 *)dst);
- if (0 && code != TCL_OK)
-  {
-   LangDebug("%s:%.*s => %d with %d bytes\n",Tcl_GetEncodingName(encoding),
-             srcLen,src,code,*dstWrotePtr);
-  }
  return code;
 }
 
+int
+Tcl_ExternalToUtf (Tcl_Interp * interp,
+				Tcl_Encoding encoding, CONST char * src,
+				int srcLen, int flags,
+				Tcl_EncodingState * statePtr, char * dst,
+				int dstLen, int * srcReadPtr,
+				int * dstWrotePtr, int * dstCharsPtr)
+{
+ return CallEncode(interp,encoding,src,srcLen,flags,statePtr,dst,dstLen,
+                   srcReadPtr,dstWrotePtr,dstCharsPtr,"decode");
+}
+
+int
+Tcl_UtfToExternal(Tcl_Interp * interp,
+				Tcl_Encoding encoding, CONST char * src,
+				int srcLen, int flags,
+				Tcl_EncodingState * statePtr, char * dst,
+				int dstLen, int * srcReadPtr,
+				int * dstWrotePtr, int * dstCharsPtr)
+{
+ return CallEncode(interp,encoding,src,srcLen,flags,statePtr,dst,dstLen,
+                   srcReadPtr,dstWrotePtr,dstCharsPtr,"encode");
+}
+
+
+
 char *
 Tcl_UtfToExternalDString(Tcl_Encoding encoding, CONST char * src,
                          int srcLen, Tcl_DString * dsPtr)
@@ -757,7 +810,7 @@ Tcl_UtfToExternalDString(Tcl_Encoding encoding, CONST char * src,
  SV *sv;
  char *s    = "";
  STRLEN len = 0;
-
+ SV *fallback = get_sv("Tk::encodeFallback",0);
  Tcl_DStringInit(dsPtr);
 
  if (!encoding)
@@ -768,6 +821,7 @@ Tcl_UtfToExternalDString(Tcl_Encoding encoding, CONST char * src,
   srcLen = strlen(src);
  if (srcLen)
   {
+   int count;
    ENTER;
    SAVETMPS;
    PUSHMARK(sp);
@@ -776,13 +830,21 @@ Tcl_UtfToExternalDString(Tcl_Encoding encoding, CONST char * src,
    sv_setpvn(sv,src,srcLen);
    sv_maybe_utf8(sv);
    XPUSHs(sv_2mortal(sv));
+   XPUSHs(fallback);
    PUTBACK;
-   perl_call_method("encode",G_SCALAR);
+   count = perl_call_method("encode",G_SCALAR);
    SPAGAIN;
-   sv = POPs;
-   PUTBACK;
-   if (SvPOK(sv))
-    s  = SvPV(sv,len);
+   if (count > 0)
+    {
+     sv = POPs;
+     PUTBACK;
+     if (sv && SvPOK(sv))
+      s  = SvPV(sv,len);
+    }
+   else
+    {
+     LangDebug("Encode did not return a value:%s\n",SvPV_nolen(ERRSV));
+    }
    Tcl_DStringAppend(dsPtr,s,len);
    FREETMPS;
    LEAVE;
